// Soubor GO2.CS
// Implementace tdy GrafickObjekt a jeho potomk
// Pouit vrtulnch metod vede k polymorfnmu chovn

using System;
using System.Collections.Generic;

namespace grafika
{
    // Vtov typ pro barvy
    public enum Barva { erven, Zelen, Modr, Bl }

    //spolen pedek implementuje spolen vlastnosti vdch grafickch objekt
    public class GrafickObjekt
    {
        private Barva barva;
        protected bool Nakreslen { set; get; } = false;

        public Barva Barva      // M vlastnost barva, pi zmn barvy se pekresl
        {
            set
            {
                barva = value;
                if (Nakreslen) Nakresli();
            }
            get => barva;
        }
        public GrafickObjekt(Barva barva = Barva.Bl) { Barva = barva; }
        public virtual void Nakresli() // "Nakresl" obecn grafick objekt
        {
            Nakreslen = true;
            Console.WriteLine("GrafickObjekt, barva " + Barva);
        }
    }


    // Bod zdd barvu 
    // Metodu Nakresli nutno pedefinovat (bod se kresl jinak ne obecn grafick objekt)
    // Aby bylo mono zachzet s Bodem jako s obecnm grafickm objektem, mus bt 
    // metoda Nakresli virtuln
    public class Bod : GrafickObjekt
    {
        public int X { get; set; }    
        public int Y { get; set; }        
        public override void Nakresli()
        {
            Nakreslen = true;
            Console.WriteLine("Kreslm Bod, barva " + Barva + ", (" + X + ", " + Y +
                        ")");
        }
        public Bod(int x, int y, Barva barva): base(barva) => (X, Y) = (x, y);
    }


    // Zakroukovan bod zddil Barvu po Grafickm objektu (prostednictvm Bodu) 
    // a souadnice od Bodu
    public class ZakroukovanBod : Bod
    {
        public ZakroukovanBod(int x, int y, Barva barva) : base(x, y, barva) { }
        private void NakresliKrouek()
        {
            Console.WriteLine("Kreslm krouek");
        }
        public override void Nakresli()
        {       
            base.Nakresli();            // Vol metodu pedka
            NakresliKrouek();
        }
    }


    // seka je urena dvma krajnmi body
    public class seka : GrafickObjekt
    {
        public Bod Potek { get; set; }        
        public Bod Konec { get; set; }
        public override void Nakresli()// "Kreslen" seky
        {
            Console.WriteLine("Kreslm seku, barva " + Barva + ", krajn body:");
            Potek.Nakresli();
            Konec.Nakresli();
        }
        public seka(int x1, int y1, int x2, int y2, Barva barva)
            : base(barva)              // Konstruktor pedka
        {
            Potek = new Bod(x1, y1, barva);
            Konec = new Bod(x2, y2, barva);
        }
    }

    // Krunice je urena stedem, co je Bod, a polomrem
    public class Krunice : GrafickObjekt
    {
        protected Bod sted;
        protected int r;

        public int R { get; set; }

        public Bod Sted { get; set; }
        

        public override void Nakresli()
        {
            Console.WriteLine("Kreslm Krunici, barva " +
                       Barva + ", polomr " + R + ", sted: ");
            Sted.Nakresli();
        }

        public Krunice(int x, int y, int r, Barva barva) : base(barva)
        {
            Sted = new Bod(x, y, barva);
            R = r;
        }

    }

    public class Obrzek : GrafickObjekt
    {
        private readonly List<GrafickObjekt> sloky = new List<GrafickObjekt>();
        public Obrzek() { }
        public void Pidej(GrafickObjekt go) => sloky.Add(go);
        public override void Nakresli()
        {
            foreach(var go in sloky)
            {
                go.Nakresli();
            }
        }
    }
    
    class Program
    {
        public static void Main()
        {
            Obrzek obrzek = new Obrzek();
            obrzek.Pidej(new seka(1, 2, 3, 4, Barva.Modr));
            obrzek.Pidej(new Krunice(8, 9, 10, Barva.Zelen));
            obrzek.Pidej(new ZakroukovanBod(6, 9, Barva.erven));
            obrzek.Nakresli();
        }
    }
}