﻿using System.Collections;
using System.Collections.Generic;

namespace SeznamCS
{
    class Seznam<T>: IEnumerable<T>
    {
        private Prvek hlava = null;
        private Prvek zarážka = null;
        public IEnumerator<T> GetEnumerator() => new Enumerátor(this);
        IEnumerator IEnumerable.GetEnumerator() => new Enumerátor(this);
        public Seznam() => hlava = zarážka = new Prvek();
        public Seznam(IEnumerable<T> kolekce)
            :this()
        {
            foreach(var prvek in kolekce)
            {
                VložNaKonec(prvek);
            }
        }
        public Seznam(params T[] prvky)
            :this((IEnumerable<T>) prvky)
        {  }
        
        public bool Prázdný => hlava == zarážka;
        public int Velikost { get; private set; } = 0;
        public void VložNaKonec(T co)
        {
            zarážka.Data = co;
            zarážka.Další = new Prvek();
            zarážka = zarážka.Další;
            Velikost += 1;
        }
        public bool Najdi(T co, out IEnumerator<T> nalezený)
        {
            nalezený = Enumerátor.NaNic;
            if (Prázdný)
            {
                return false;
            }
            for (var prvek = hlava; prvek != zarážka; prvek = prvek.Další)
            {
                if(prvek.Data.Equals(co))
                {
                    nalezený = new Enumerátor(this, prvek);
                    return true;
                }
            }
            return false;
        }

        public void ZměňHodnotu(IEnumerator<T> ukazatelNaPrvek, T nováHodnota)
        {
            ((Enumerátor)ukazatelNaPrvek).Aktuální.Data = nováHodnota;
        }
        public void VložNaZačátek(T co)
        {
            hlava = new Prvek(co, hlava);
            Velikost++;
        }
        public void VložZa(T co, IEnumerator<T> kam)
        {
            Prvek zaKterý = ((Enumerátor)kam).Aktuální;
            zaKterý.Další = new Prvek(co, zaKterý.Další);
            Velikost++ ;
        }

        public void OdstraňPrvní()
        {
            if (Prázdný)
            {
                throw new System.InvalidOperationException("Odstraňování z prázdného seznamu");
            }
            hlava = hlava.Další;
            Velikost--;
        }
        public void Odstraň(IEnumerator<T> který)
        {
            Prvek kterýOdstranit = ((Enumerátor)který).Aktuální;
            if (kterýOdstranit == hlava)
            {
                hlava = hlava.Další;
            }
            else if (((Enumerátor)který).Poslední)
            {
                zarážka = kterýOdstranit;
                zarážka.Další = null;
            }
            else
            {
                kterýOdstranit.Data = kterýOdstranit.Další.Data;
                kterýOdstranit.Další = kterýOdstranit.Další.Další;
            }
            Velikost -= 1;
        }
        public void Vyprázdni()
        {
            hlava = zarážka;
            Velikost = 0;
        }

        private class Prvek
        {
            public T Data { get; set; }
            public Prvek Další { get; set; }
            public Prvek(T data = default(T), Prvek další = null)
            {
                (Data, Další) = (data, další);
            }
        }
        private class Enumerátor : IEnumerator<T>
        {
            Seznam<T> seznam = null;
            public Prvek Aktuální { get; set; } = null;
            public T Current => Aktuální.Data;
            object IEnumerator.Current => Aktuální.Data;

            public void Dispose() { }
            public bool MoveNext()
            {
                if (seznam.Prázdný || Poslední)
                {
                    return false;
                }
                Aktuální = (Aktuální == null) ? seznam.hlava : Aktuální.Další;
                return true;
            }
            public Enumerátor(Seznam<T> seznam, Prvek prvek = null) => (this.seznam, this.Aktuální) = (seznam, prvek);
            public void Reset() => Aktuální = null;
            public bool Poslední => (Aktuální != null) && (Aktuální.Další == seznam.zarážka);

            public override bool Equals(object obj)
            {
                var enumerátor = obj as Enumerátor;
                return (enumerátor != null) && (seznam == enumerátor.seznam) 
                    && (Aktuální == enumerátor.Aktuální);  
            }

            public static readonly Enumerátor NaNic = new Enumerátor(null);
        }
    }
    
}
