// Pklad na peten opertor true a false
// Aby bylo mono instance rozumn pout v podmnkch ap., 
// je teba petit jet dal logick opertory
//


using System;

namespace truefalsenula
{
    public class TrojhodnotovBool
    {
        // Mon hodnoty
        public static readonly TrojhodnotovBool Nula = new TrojhodnotovBool(0);
        public static readonly TrojhodnotovBool False = new TrojhodnotovBool(-1);
        public static readonly TrojhodnotovBool True = new TrojhodnotovBool(1);

        private sbyte hodnota;

        // Konstruktor
        private TrojhodnotovBool(int hodnota) => this.hodnota = (sbyte)hodnota;

        // Jednoduch vlastnosti pro zjitn hodnoty
        public bool JeNula => hodnota == 0;
        public bool JeFalse => hodnota < 0;
        public bool JeTrue => hodnota > 0;

        // Konverzn opertor bool->MjDBBool
        public static implicit operator TrojhodnotovBool(bool x)
        {
            return x ? True : False;
        }

        // Konverzn opertor  MjDBBool -> bool
        public static explicit operator bool(TrojhodnotovBool x)
        {
            if (x.hodnota == 0)
            {
                throw new InvalidOperationException
                                    ("Nelze provst konverzi typu TrojhodnotovBool na bool. TrojhodnotovBool==Nula");
            }

            return x.hodnota > 0;
        }

        // Opertory rovnosti a nerovnosti
        public static TrojhodnotovBool operator ==(TrojhodnotovBool x, TrojhodnotovBool y)
        {
            if (x.hodnota == 0 || y.hodnota == 0) return Nula;
            return x.hodnota == y.hodnota ? True : False;
        }

        public static TrojhodnotovBool operator !=(TrojhodnotovBool x, TrojhodnotovBool y)
        {
            if (x.hodnota == 0 || y.hodnota == 0) return Nula;
            return x.hodnota != y.hodnota ? True : False;
        }

        // Opertor negace
        public static TrojhodnotovBool operator !(TrojhodnotovBool x)
        {
            return new TrojhodnotovBool(-x.hodnota);
        }

        // Logick opertory AND a OR
        public static TrojhodnotovBool operator &(TrojhodnotovBool x, TrojhodnotovBool y)
        {
            return new TrojhodnotovBool(x.hodnota < y.hodnota ? x.hodnota : y.hodnota);
        }

        public static TrojhodnotovBool operator |(TrojhodnotovBool x, TrojhodnotovBool y)
        {
            return new TrojhodnotovBool(x.hodnota > y.hodnota ? x.hodnota : y.hodnota);
        }

        // Opertory true a false
        public static bool operator true(TrojhodnotovBool x) => x.hodnota > 0;

        public static bool operator false(TrojhodnotovBool x) => x.hodnota < 0;
        public override bool Equals(object obj)
        {
            if(obj is TrojhodnotovBool)
            {
                var thb = obj as TrojhodnotovBool;
                return hodnota == thb.hodnota;
            }
            return false;
        }
        public override int GetHashCode() => hodnota;
    }

    class Program
    {
        // Analyzuje hodnotu tstavov promnn 
        private static void CoToJe(TrojhodnotovBool b)
        {
            if (b)  // je TRUE ?
            {
                Console.WriteLine("Hodnota = True");
            }
            else if (!b) // je FALSE ?
            {
                Console.WriteLine("Hodnota = False");
            }
            else // je to Nula
            {
                Console.WriteLine("Hodnota = Nula");
            }
        }

        static void Main()
        {
            TrojhodnotovBool b = TrojhodnotovBool.True;
            CoToJe(b);

            b = TrojhodnotovBool.False;
            CoToJe(b);

            b = TrojhodnotovBool.Nula;
            CoToJe(b);

            TrojhodnotovBool a = true;
            TrojhodnotovBool c = TrojhodnotovBool.Nula;

            c = a && c;
            CoToJe(c);
            if (a && b)
            {
                CoToJe(c);
            }

            c = a || b;
            CoToJe(c);
        }
    }
}

