using System;
using System.Collections.Generic;
using System.Linq;

namespace nulovateln
{

    class Program
    {
        public class Optional<T>
        {
            public bool HasValue { get; private set; }

            private T value;
            public T Value
            {
                get
                {
                    if (HasValue) return value;
                    else throw new InvalidOperationException();
                }
            }

            public Optional(T value) => (this.value, HasValue) = (value, true);
            public Optional() => (HasValue, value) = (false, default(T));

            public static explicit operator T(Optional<T> optional) => optional.Value;
            public static implicit operator Optional<T>(T value) => new Optional<T>(value);

            public override bool Equals(object obj)
            {
                return obj is Optional<T> ? this.Equals((Optional<T>)obj) : false;
            }
            public bool Equals(Optional<T> druh)
            {
                return HasValue && druh.HasValue ? object.Equals(value, druh.value) : HasValue == druh.HasValue;
            }
        }

        static Optional<int> Hodnota(int i)
        {
            return i == 0 ? new Optional<int>() : i;
        }

        static Optional<string> Hodnota(string i)
        {
            return i == null ? new Optional<string>() : i;
        }

        static void Main(string[] args)
        {
            var x = Hodnota(5);
            if(x.HasValue)
            {
                Console.WriteLine(x.Value);
            }
            else
            {
                Console.WriteLine("smla");
            }
            
            var y = Hodnota(null);
            if (y.HasValue)
            {
                Console.WriteLine(y.Value);
            }
            else
            {
                Console.WriteLine("smla");
            }
        }
    }
}
