using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace komplexn
{
    struct Komplex
    {
        public double Re { get; set; }
        public double Im { get; set; }
        public Komplex(double re, double im = 0) => (Re, Im) = (re, im);
        public static readonly Komplex NULA = new Komplex(0);
        public static readonly Komplex i = new Komplex(0, 1);
        public override string ToString() => Re + " + " + Im + ".i";
        public void Deconstruct(out double re, out double im) => (re, im) = (Re, Im);

        public override bool Equals(object obj)
        {
            return obj is Komplex z ? (Re == z.Re) && (Im == z.Im) : false;
        }
        public override int GetHashCode()
        {
            return Re.GetHashCode() ^ Im.GetHashCode();
        }

        public static implicit operator Komplex((double re, double im) v) => new Komplex(v.re, v.im);
        public static implicit operator Komplex(double re)=>(re, 0);
        public static explicit operator double(Komplex z)
        {
            if (z.Im != 0)
            {
                throw new ArgumentException("Nenulov imaginrn st pi konevrzi na double");
            }

            return z.Re;
        }
        public static Komplex operator +(Komplex x) => x;
        public static Komplex operator -(Komplex x) => (-x.Re, -x.Im);
        public static Komplex operator +(Komplex z, Komplex u) => (z.Re + u.Re, z.Im + u.Im);
        public static Komplex operator -(Komplex z, Komplex u) => (z.Re - u.Re, z.Im - u.Im);
        public static Komplex operator *(Komplex z, Komplex u) => (z.Re * u.Re - z.Im * u.Im, z.Re * u.Im + u.Re * z.Im);
        public static Komplex operator /(Komplex z, double x) => (z.Re / x, z.Im / x);
        public static Komplex operator /(Komplex z, Komplex u)
        {
            var pomoc = u.Sdruen();
            return (z * pomoc) / (u * pomoc).Re;
        }

        public static bool operator ==(Komplex a, Komplex b) => a.Equals(b);
        public static bool operator !=(Komplex a, Komplex b) => !(a == b);
        public Komplex Sdruen() => (Re, -Im);
        public double this[int i]
        {
            get
            {
                switch(i)
                {
                    case 1: return Re;
                    case 2: return Im;
                    default: throw new IndexOutOfRangeException("Index mus bt 1 nebo 2");
                }
            }
            set
            {
                switch (i)
                {
                    case 1:
                        Re = value;
                        return;
                    case 2:
                        Im = value;
                        return;
                    default: throw new IndexOutOfRangeException("Index mus bt 1 nebo 2");
                }
            }
        }
    }
}
