using System;
using System.Collections.Generic;
using System.Linq;

namespace kontakty
{
    public enum TypKontaktu { Soukrom, Pracovn };
    class Kontakt
    {
        public string Jmno { get; set; }
        public string Pjmen { get; set; }
        public string Telefon { set; get; }
        public TypKontaktu Typ { set; get; }
        public int RokSeznmen { get; set; }
        public Kontakt(string jmno, string pjmen, string telefon, TypKontaktu typ, int rok) => (Jmno, Pjmen, Telefon, Typ, RokSeznmen) = (jmno, pjmen, telefon, typ, rok);
        public override string ToString()
        {
            return $"{Pjmen}, {Jmno} ({Typ}: {Telefon}, od r. {RokSeznmen})";
        }
    }

    class SkupinyPodleDesetilet : IEqualityComparer<int>
    {
        public bool Equals(int x, int y)
        {
            return x / 10 == y / 10;
        }
        public int GetHashCode(int x)
        {
            return (x / 10) * 10;
        }
    }


    class Program
    {
        private static List<Kontakt> kontakty = new List<Kontakt>();
        private static void Pprava()
        {
            kontakty.Add(new Kontakt("Josef", "Novk", "523856581", TypKontaktu.Pracovn, 2015));
            kontakty.Add(new Kontakt("Jan", "Vopika", "123456789", TypKontaktu.Pracovn, 1999));
            kontakty.Add(new Kontakt("Jan", "Novk", "852741963", TypKontaktu.Soukrom, 2018));
            kontakty.Add(new Kontakt("Iva", "Klapalov", "121454585", TypKontaktu.Soukrom, 2019));
            kontakty.Add(new Kontakt("Vclav", "Maek", "321654987", TypKontaktu.Pracovn, 2020));
            kontakty.Add(new Kontakt("Karel", "Mrek", "987654123", TypKontaktu.Soukrom, 2015));
            kontakty.Add(new Kontakt("Hana", "Mazan", "326598741", TypKontaktu.Pracovn, 2017));
            kontakty.Add(new Kontakt("Josef", "Novk", "313265649", TypKontaktu.Pracovn, 2014));

        }

        private static void Vypi<T>(IEnumerable<T> data)
        {
            foreach (var daj in data)
            {
                Console.WriteLine(daj);
            }
            Console.WriteLine("---");
        }

        static void VypiPoSkupinch<T, Kl>(IEnumerable<IGrouping<T, Kl>> dotaz)
        {
            foreach (var skupina in dotaz)
            {
                Console.WriteLine(skupina.Key);
                Vypi(skupina);
            }
            Console.WriteLine("---");
        }
        static void Main(string[] args)
        {
            Pprava();
            var dotaz = kontakty.GroupBy(kontakt => kontakt.RokSeznmen, new SkupinyPodleDesetilet());
            VypiPoSkupinch(dotaz);

            var dotaz1 = from kontakt in kontakty
                            group kontakt by (kontakt.RokSeznmen / 10) * 10;
            VypiPoSkupinch(dotaz1);
        }

    }
}
