using System;
using System.Collections;
using System.Collections.Generic;

namespace SeznamCS
{
    class Seznam<T>: IEnumerable<T>
    {
        private Prvek hlava = null;
        private Prvek zarka = null;
        public IEnumerator<T> GetEnumerator() => new Enumertor(this);
        IEnumerator IEnumerable.GetEnumerator() => new Enumertor(this);
        public Seznam() => hlava = zarka = new Prvek();
        public Seznam(IEnumerable<T> kolekce)
            :this()
        {
            foreach(var prvek in kolekce)
            {
                VloNaKonec(prvek);
            }
        }
        public Seznam(params T[] prvky)
            :this((IEnumerable<T>) prvky)
        {  }
        
        public bool Przdn => hlava == zarka;
        public int Velikost { get; private set; } = 0;
        public void VloNaKonec(T co)
        {
            zarka.Data = co;
            zarka.Dal = new Prvek();
            zarka = zarka.Dal;
            Velikost += 1;
        }
        public bool Najdi(T co, out IEnumerator<T> nalezen)
        {
            nalezen = Enumertor.NaNic;
            if (Przdn)
            {
                return false;
            }
            for (var prvek = hlava; prvek != zarka; prvek = prvek.Dal)
            {
                if(prvek.Data.Equals(co))
                {
                    nalezen = new Enumertor(this, prvek);
                    return true;
                }
            }
            return false;
        }

        public void ZmHodnotu(IEnumerator<T> ukazatelNaPrvek, T novHodnota)
        {
            ((Enumertor)ukazatelNaPrvek).Aktuln.Data = novHodnota;
        }
        public void VloNaZatek(T co)
        {
            hlava = new Prvek(co, hlava);
            Velikost++;
        }
        public void VloZa(T co, IEnumerator<T> kam)
        {
            Prvek zaKter = ((Enumertor)kam).Aktuln;
            zaKter.Dal = new Prvek(co, zaKter.Dal);
            Velikost++ ;
        }

        public void OdstraPrvn()
        {
            if (Przdn)
            {
                throw new System.InvalidOperationException("Odstraovn z przdnho seznamu");
            }
            hlava = hlava.Dal;
            Velikost--;
        }
        public void Odstra(IEnumerator<T> kter)
        {
            Prvek kterOdstranit = ((Enumertor)kter).Aktuln;
            if (kterOdstranit == hlava)
            {
                hlava = hlava.Dal;
            }
            else if (((Enumertor)kter).Posledn)
            {
                zarka = kterOdstranit;
                zarka.Dal = null;
            }
            else
            {
                kterOdstranit.Data = kterOdstranit.Dal.Data;
                kterOdstranit.Dal = kterOdstranit.Dal.Dal;
            }
            Velikost -= 1;
        }
        public void Vyprzdni()
        {
            hlava = zarka;
            Velikost = 0;
        }

        private Prvek NajdiItPrvek(int i)
        {
            if ((i < 0) || (i >= Velikost))
            {
                throw new IndexOutOfRangeException("Index mimo rozsah");
            }
            var vsledek = hlava;
            for (int j = 0; j < i; j++)
            {
                vsledek = vsledek.Dal;
            }
            return vsledek;
        }

        public T this[int i]
        {
            get => NajdiItPrvek(i).Data;
            set => NajdiItPrvek(i).Data = value;
        }

        private class Prvek
        {
            public T Data { get; set; }
            public Prvek Dal { get; set; }
            public Prvek(T data = default(T), Prvek dal = null)
            {
                (Data, Dal) = (data, dal);
            }
        }
        private class Enumertor : IEnumerator<T>
        {
            Seznam<T> seznam = null;
            public Prvek Aktuln { get; set; } = null;
            public T Current => Aktuln.Data;
            object IEnumerator.Current => Aktuln.Data;

            public void Dispose() { }
            public bool MoveNext()
            {
                if (seznam.Przdn || Posledn)
                {
                    return false;
                }
                Aktuln = (Aktuln == null) ? seznam.hlava : Aktuln.Dal;
                return true;
            }
            public Enumertor(Seznam<T> seznam, Prvek prvek = null) => (this.seznam, this.Aktuln) = (seznam, prvek);
            public void Reset() => Aktuln = null;
            public bool Posledn => (Aktuln != null) && (Aktuln.Dal == seznam.zarka);

            public override bool Equals(object obj)
            {
                var enumertor = obj as Enumertor;
                return (enumertor != null) && (seznam == enumertor.seznam) 
                    && (Aktuln == enumertor.Aktuln);  
            }

            public static readonly Enumertor NaNic = new Enumertor(null);
        }
    }
    
}
