// Soubor GO3.CS
// Implementace tdy GrafickObjekt a jeho potomk
// Pouit vrtulnch metod vede k polymorfnmu chovn

using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace grafika
{
    // Vtov typ pro barvy
    public enum Barva { erven, Zelen, Modr, Bl }

    //spolen pedek implementuje spolen vlastnosti vdch grafickch objekt
    [Serializable]
    public abstract class GrafickObjekt : ICloneable
    {
        private Barva barva;
        protected bool Nakreslen { set; get; } = false;

        public Barva Barva      // M vlastnost barva, pi zmn barvy se pekresl
        {
            set
            {
                barva = value;
                if (Nakreslen) Nakresli();
            }
            get => barva;
        }
        public GrafickObjekt(Barva barva = Barva.Bl) { Barva = barva; }
        public abstract void Nakresli(); // "Nakresl" obecn grafick objekt
        public virtual object Clone() => MemberwiseClone();
    }


    // Bod zdd barvu 
    // Metodu Nakresli nutno pedefinovat (bod se kresl jinak ne obecn grafick objekt)
    // Aby bylo mono zachzet s Bodem jako s obecnm grafickm objektem, mus bt 
    // metoda Nakresli virtuln
    [Serializable]
    public class Bod : GrafickObjekt
    {
        public int X { get; set; }
        public int Y { get; set; }
        public override void Nakresli()
        {
            Nakreslen = true;
            Console.WriteLine("Kreslm Bod, barva " + Barva + ", (" + X + ", " + Y +
                        ")");
        }
        public Bod(int x, int y, Barva barva) : base(barva) => (X, Y) = (x, y);
    }


    // Zakroukovan bod zddil Barvu po Grafickm objektu (prostednictvm Bodu) 
    // a souadnice od Bodu
    [Serializable]
    public class ZakroukovanBod : Bod
    {
        public ZakroukovanBod(int x, int y, Barva barva) : base(x, y, barva) { }
        private void NakresliKrouek()
        {
            Console.WriteLine("Kreslm krouek");
        }
        public override void Nakresli()
        {
            base.Nakresli();            // Vol metodu pedka
            NakresliKrouek();
        }
    }


    // seka je urena dvma krajnmi body
    [Serializable]
    public class seka : GrafickObjekt
    {
        private Bod potek;
        private Bod konec;
        public Bod Potek
        {
            get => potek;
            set => potek = (Bod)value.Clone();
        }
        public Bod Konec
        {
            get => konec;
            set => konec = (Bod)value.Clone();
        }
        public override void Nakresli()// "Kreslen" seky
        {
            Console.WriteLine("Kreslm seku, barva " + Barva + ", krajn body:");
            Potek.Nakresli();
            Konec.Nakresli();
        }
        public seka(int x1, int y1, int x2, int y2, Barva barva)
            : base(barva)              // Konstruktor pedka
        {
            potek = new Bod(x1, y1, barva);
            konec = new Bod(x2, y2, barva);
        }

        public override object Clone()
        {
            seka vsledek = (seka)base.Clone();
            vsledek.Potek = (Bod)this.Potek.Clone();
            vsledek.Konec = (Bod)this.Konec.Clone();
            return vsledek;
        }
    }

    // Krunice je urena stedem, co je Bod, a polomrem
    [Serializable]
    public class Krunice : GrafickObjekt
    {
        protected Bod sted;
        protected int r;

        public int R { get; set; }

        public Bod Sted
        {
            get => sted;
            set => sted = (Bod)value.Clone();
        }


        public override void Nakresli()
        {
            Console.WriteLine("Kreslm Krunici, barva " +
                        Barva + ", polomr " + R + ", sted: ");
            Sted.Nakresli();
        }

        public Krunice(int x, int y, int r, Barva barva) : base(barva)
        {
            Sted = new Bod(x, y, barva);
            R = r;
        }

        public override object Clone()
        {
            Krunice vsledek = (Krunice)base.Clone();
            vsledek.Sted = (Bod)this.Sted.Clone();
            return vsledek;
        }

    }
    [Serializable]
    public class Obrzek : GrafickObjekt
    {
        private readonly List<GrafickObjekt> sloky = new List<GrafickObjekt>();
        public Obrzek() { }
        public void Pidej(GrafickObjekt go) => sloky.Add(go);
        public override void Nakresli()
        {
            foreach (var go in sloky)
            {
                go.Nakresli();
            }
        }
        public void Ulo(string jmnoSouboru)
        {
            using (FileStream proud = new FileStream(jmnoSouboru, FileMode.Create,FileAccess.Write))
            {
                BinaryFormatter serializtor = new BinaryFormatter();
                serializtor.Serialize(proud, this);
            }
        }
        public static Obrzek Nati(string jmnoSouboru)
        {
            using (FileStream proud = new FileStream(jmnoSouboru, FileMode.Open, FileAccess.Read))
            {
                BinaryFormatter serializtor = new BinaryFormatter();
                return serializtor.Deserialize(proud) as Obrzek;
            }
        }
    }

    class Program
    {
        static string jmno = "obrzek.obr";
        public static void Main()
        {
            //Obrzek obrzek = new Obrzek();
            //seka seka = new seka(1, 1, 2, 2, Barva.Modr);
            //obrzek.Pidej(seka);
            //Bod bod = new Bod(1, 5, Barva.Zelen);
            //obrzek.Pidej(bod);
            //obrzek.Nakresli();
            //obrzek.Ulo(jmno);
            Obrzek obrzek = Obrzek.Nati(jmno);
            obrzek.Nakresli()
;        }
    }
}