﻿using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.IO;

namespace ProblémNDam
{
    public partial class Okno : Form
    {
        private int kolik = 0;
        private Řešitel řešitel = null;
        bool nalezeno = false;
        int[] řešení;
        public Okno()
        {
            InitializeComponent();
        }

        private void PřipravŘešitele()
        {
            kolik = (int)číselníkPočetDam.Value;
            řešitel = new Řešitel(kolik);
        }

        private void MenuNová_Click(object sender, EventArgs e)
        {
            PřipravŘešitele();
            NastavTitulek(kolik, řešitel.PočetNalezenýchŘešení);
            (menuDalší.Enabled, tlačítkoDalší.Enabled, nalezeno) = (true, true, false);
            panel.Invalidate();
        }

        private void NastavTitulek(int kolik, int početŘešení)
        {
            Text = $"Problém {kolik} dam, je zobrazeno {početŘešení}. řešení";
        }

        private static void Neimplementováno()
        {
            MessageBox.Show("Tato možnost zatím není implementována.", "Upozornění", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void MenuDalší_Click(object sender, EventArgs e)
        {
            nalezeno = řešitel.NajdiDalšíŘešení(out řešení);
            if(!nalezeno)
            {
                MessageBox.Show("Další řešení neexistují.", "To je vše", MessageBoxButtons.OK, MessageBoxIcon.Information);
                (menuDalší.Enabled, tlačítkoDalší.Enabled) = (false, false);
            }
            panel.Invalidate();
            NastavTitulek(kolik, řešitel.PočetNalezenýchŘešení);
        }

        private void MenuKonec_Click(object sender, EventArgs e)
        {
            Close();
            Application.Exit();
        }

        private void MenuOProgramu_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Úloha N dam\n© Piškvorky software, s.b.r. 2020", "O programu");
        }

        private void Okno_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (MessageBox.Show("Opravdu chceš skončit?", "Dotaz", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                e.Cancel = true;
            }

        }

        private void Panel_Paint(object sender, PaintEventArgs e)
        {
            Graphics grafika = e.Graphics;
            if (nalezeno)
            {
                Pen pero = new Pen(Color.Black)
                {
                    Width = 1
                };
                var (deltax, deltay) = (panel.Width / kolik, panel.Height / kolik);
                NakresliŠachovnici(grafika, pero, deltax, deltay);
                NakresliDámy(grafika, pero, deltax, deltay);
            }
            else
            {
                grafika.Clear(Color.White);
            }
        }

        private void NakresliDámy(Graphics grafika, Pen pero, int deltax, int deltay)
        {
            pero.Width = 6;
            pero.Color = Color.Red;
            for (int i = 0; i < kolik; i++)
            {
                grafika.DrawEllipse(pero, i * deltax + deltax / 6, řešení[i] * deltay + deltay / 6, 2 * deltax / 3, 2 * deltay / 3);
            }
        }

        private void NakresliŠachovnici(Graphics grafika, Pen pero, int deltax, int deltay)
        {
            for (int i = 1; i < kolik; i++)
            {
                grafika.DrawLine(pero, 0, i * deltay, panel.Width, i * deltay);
                grafika.DrawLine(pero, i * deltax, 0, i * deltax, panel.Height);
            }
        }
        private void MenuVšechna_Click(object sender, EventArgs e)
        {
            (bool, bool, bool, bool) stav = (menuNová.Enabled, menuDalší.Enabled, menuVšechna.Enabled, tlačítkoDalší.Enabled);
            (menuNová.Enabled, menuDalší.Enabled, menuVšechna.Enabled) = (false, false, false);
            PřipravŘešitele();
            MessageBox.Show($"Úloha {kolik} dam má celkem {řešitel.Řešení.Count()} řešení, \npočet zkoušek políček: {řešitel.PočetVolání}", "Výsledek");
            (menuNová.Enabled, menuDalší.Enabled, menuVšechna.Enabled, tlačítkoDalší.Enabled) = stav;
        }

        private void Okno_Resize(object sender, EventArgs e)
        {
            panel.Invalidate();
        }

        private void MenuUložit_Click(object sender, EventArgs e)
        {
            if(dialogUložení.ShowDialog() == DialogResult.OK)
            {
                PřipravŘešitele();
                using (var výstup = new StreamWriter(dialogUložení.FileName))
                {
                    foreach(var řádka in řešitel.Řešení)
                    {
                        foreach(var i in řádka)
                        {
                            výstup.Write($"{i} ");
                        }
                        výstup.WriteLine();
                    }
                }
            }
        } 
    }
}
