﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace ProblémNDam
{
    public class Řešitel
    {
        private readonly int N;
        private readonly int[] řešení;
        private int[] p = null;
        public int PočetNalezenýchŘešení { get; private set; } = 0;
        public long PočetVolání { get; private set; } = 0;

        public IEnumerable<int[]> Řešení
        {
            get
            {
                while (NajdiDalšíŘešení(out p))
                {
                    yield return p;
                }
            }
        }


        // Konstruktor řešitele pro úlohu n dam
        public Řešitel(int n)
        {
            N = n;
            řešení = new int[n];
            for (int i = 0; i < n; i++)
            {
                řešení[i] = -1;
            }
        }
        public bool NajdiDalšíŘešení(out int[] výsledek)
        {
            výsledek = PočetNalezenýchŘešení == 0 ? ŘešOdSloupce(0) : ŘešOdSloupce(N - 1);
            if (výsledek != null)
            {
                PočetNalezenýchŘešení++;
            }
            return výsledek != null;
        }

        public void NajdiVšechnaŘešení()
        {
            int[] p = null;
            while (NajdiDalšíŘešení(out p))
                ;
        }

        private bool Bezpečná(int k, int m)// k-tá řádka, m-tý sloupec
        {
            Application.DoEvents();
            PočetVolání++;
            if (m == 0)
            {
                return true;    // V prvním sloupci lze umístit kamkoli
            }
            var (výsledek, j) = (true, 0);
            while (výsledek && (j < m))         // Projdi všechny předchozí sloupce
            {
                výsledek = (řešení[j] != k) &&      // stejná řádka?
                    (řešení[m - j - 1] != k - j - 1) && // stejná diagonála?
                    (řešení[m - j - 1] != k + j + 1);
                if (!výsledek)
                {
                    return false;
                }
                j++;
            }
            return true;
        }

        public void Výpis()
        {
            Console.Write("(");
            for (int i = 0; i < N; i++)
            {
                Console.Write(řešení[i] + (i < N - 1 ? ", " : ""));
            }
            Console.WriteLine(")");
        }

        // Najde další možné umístění v m-tém sloupci, začne za k-tou řádkou
        private int NajdiDalšíBezpečnouŘádku(int řádka, int sloupec)   
        {

            while (++řádka < N)
            {
                if (Bezpečná(řádka, sloupec))
                {
                    break;
                }
            }
            return řádka;                           // Neuspěje-li, vrátí N
        }

        // Začne hledat další řešení počínaje m-tým sloupcem
        private int[] ŘešOdSloupce(int m)            
        {
            int k;
            while (m > -1)
            {
                k = NajdiDalšíBezpečnouŘádku(řešení[m], m);
                if (k < N)
                {
                    řešení[m++] = k;
                    if (m == N) return řešení;
                }
                else řešení[m--] = -1;
            }
            return null;
        }

        private void NajdiŘešení(int m)
        {
            if (m == N)
            {
                //Výpis();
                PočetNalezenýchŘešení++;
            }
            else
            {
                for (int k = 0; k < N; k++)
                {
                    if (Bezpečná(k, m))
                    {
                        řešení[m] = k;
                        NajdiŘešení(m + 1);
                    }
                }
            }
            if (m < N) řešení[m] = -1;
        }
    }
}
