// Soubor DAMY.CS
// Řešení problému N dam (Gauss ~1850)
// Umístit na šachovnici s nxn poli n dam tak, aby se navzájem
// neohrožovaly.
//
// Třída Řešitel používá k nalezení všech řešení backtracking 
// Konstruktor zkonstruuje řešitele pro zadané n
// výsledek je v poli řešení uložen tako
// (šachovnice má řádky a sloupce číslovány od 0 do n-1):
// - řešení[m] == k znamená, že dáma je v m-tém sloupci v k-té řádce
//
// Metoda Bezpečné() testuje, zda je daná pozice bezpečná, tj. 
// zda dámu na něm umístěnou nebode ohrožovat jiná, umístěná dříve
//
// NajdiDalšíBezpečnouŘádku() najde další přípustnou pozici dámy v daném sloupci
//
// Řeš() najde další řešení (jedno) a skončí
//
// NajdiVšechnaŘešení() najde všechna řešení a vypíše je; 
// používá rekurzivní metodu NajdiŘešení()
// bool NajdiDalšíŘešení(out int[] výsledek) najde další řešení; vrátí false, 
// pokud neexistuje. V parametru výsledek vrátí odkaz na toto řešení.

using System;
using System.Collections.Generic;
using System.Linq;


public class Řešitel
{
    private readonly int n;
    private readonly int[] řešení;
    public int PočetŘešení { get; private set; } = 0;
    int[] p = null;
    public IEnumerable<int[]> Řešení
    {
        get
        {
            while (NajdiDalšíŘešení(out p))
            {
                yield return p;
            }
        }
    }

    public bool NajdiDalšíŘešení(out int[] výsledek)
    {
        výsledek = PočetŘešení == 0 ? ŘešOdSloupce(0) : ŘešOdSloupce(n - 1);
        if (výsledek != null)
        {
            PočetŘešení++;
        }

        return výsledek != null;
    }

    public void NajdiVšechnaŘešení()
    {
        //NajdiŘešení(0);
        int[] p = null;
        while (NajdiDalšíŘešení(out p))
            ;
    }

    

    private bool Bezpečné(int k, int m)// k-tá řádka, m-tý sloupec
    {
        if (m == 0) return true;    // V prvním sloupci lze umístit kamkoli
        var (výsledek, j) = (true, 0);
        while (výsledek && (j < m))         // Projdi všechny předchozí sloupce
        {
            výsledek = (řešení[j] != k) &&      // stejná řádka?
                (řešení[m - j - 1] != k - j - 1) && // stejná diagonála?
                (řešení[m - j - 1] != k + j + 1);
            if (!výsledek) return false;
            j++;
        }
        return true;
    }

    public void Výpis()
    {
        Console.Write("(");
        for (int i = 0; i < n; i++)
        {
            Console.Write(řešení[i] + (i < n - 1 ? ", " : ""));
        }
        Console.WriteLine(")");
    }

    private int NajdiDalšíBezpečnouŘádku(int k, int m)   // Najde další možné umístění v m-tém sloupci, začne za k-tou řádkou
    {
        while (++k < n)
        {
            if (Bezpečné(k, m))
            {
                break;
            }
        }
        return k;                           // Neuspěje-li, vrátí n
    }

    private int[] ŘešOdSloupce(int m)            // Začne hledat další řešení, počínaje m-tým sloupcem
    {
        int k;
        while (m > -1)
        {
            k = NajdiDalšíBezpečnouŘádku(řešení[m], m);
            if (k < n)
            {
                řešení[m++] = k;
                if (m == n) return řešení;
            }
            else řešení[m--] = -1;
        }
        return null;
    }

    public void NajdiŘešení(int m)
    {
        if (m == n)
        {
            //Výpis();
            PočetŘešení++;
        }
        else
        {
            for (int k = 0; k < n; k++)
            {
                if (Bezpečné(k, m))
                {
                    řešení[m] = k;
                    NajdiŘešení(m + 1);
                }
            }
        }
        if (m < n) řešení[m] = -1;
    }

    public Řešitel(int n)           // Konstruktor řešitele pro úlohu n dam
    {
        this.n = n;
        řešení = new int[n];
        for (int i = 0; i < n; i++) řešení[i] = -1;
    }
}

public class Program
{
    
    public static void Main()
    {
        int kolik = 5;
        Řešitel řešitel = new Řešitel(kolik);
        //int[] p = null;
        //řešitel.NajdiVšechnaŘešení();
        //while (řešitel.NajdiDalšíŘešení(out p))
        //{
        //    //Console.Write("(");
        //    //for (int i = 0; i < kolik; i++)
        //    //    Console.Write(p[i] + ", ");
        //    //Console.WriteLine(")");
        //    //řešitel.Výpis();
        //}
        //řešitel.NajdiŘešení(0);
        //foreach(var x in řešitel.Řešení)
        //{
        //    řešitel.Výpis();
        //}

        int délka = (new Řešitel(8)).Řešení.Count();
        Console.WriteLine($"Úloha {kolik} dam má celkem {řešitel.PočetŘešení} řešení.");

    }
}