﻿using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;
using System.IO;


namespace ProblémNDam
{
    using NPR = ProblémNDam.Properties.Resources;
    using NPS = ProblémNDam.Properties.Settings;

    public partial class Okno : Form
    {
        private int kolik = 0;
        private Řešitel řešitel = null;
        private bool nalezeno = false;
        private int[] řešení;
        private readonly Časovač časovač;

        public Okno()
        {
            InitializeComponent();
            PoužijProstředky();
            PřipravŘešitele();
            časovač = new Časovač(400)
            {
                Povolen = false
            };
            časovač.Tik += MenuDalší_Click;
        }

        private void Tiknutí(object odesílatel, EventArgs ea)
        {
            nalezeno = řešitel.NajdiDalšíŘešení(out řešení);
            if (nalezeno)
            {
                panel.Invalidate();
            }
            else
            {
                časovač.Povolen = false;
            }
        }

        private void PoužijProstředky()
        {
            Icon = NPR.Dáma;
            menuHra.Text = NPR.Hra;
            menuDalší.Text = NPR.Další;
            menuOProgramu.Text = NPR.OProgramu;
            menuNápověda.Text = NPR.Nápověda;
            menuVšechna.Text = NPR.Celkový;
            menuUložit.Text = NPR.Ulož;
            menuKonec.Text = NPR.Konec;
            menuNová.Text = NPR.Nová;
            menuAnimace.Text = NPR.Animace;
            menuSpustitAnimaci.Text = NPR.Spustit;
            menuPřerušitAnimaci.Text = NPR.Zastavit;
            tlačítkoDalší.Text = NPR.Další;
            číselníkPočetDam.Maximum = NPS.Default.Maximum;
            číselníkPočetDam.Minimum = NPS.Default.Minimum;
        }

        private void PřipravŘešitele()
        {
            kolik = (int)číselníkPočetDam.Value;
            řešitel = new Řešitel(kolik);
        }

        private void MenuNová_Click(object sender, EventArgs e)
        {
            PřipravŘešitele();
            NastavTitulek(kolik, řešitel.PočetNalezenýchŘešení);
            (menuDalší.Enabled, tlačítkoDalší.Enabled, nalezeno, menuSpustitAnimaci.Enabled) = (true, true, false, true);
            panel.Invalidate();
        }

        private void NastavTitulek(int kolik, int početŘešení)
        {
            Text = string.Format(NPR.TitulekOkna, kolik, početŘešení);
        }

        private void NastavTitulek(int kolik) => Text = string.Format(NPR.TitulekAnimace, kolik);

        private static void Neimplementováno()
        {
            MessageBox.Show("Tato možnost zatím není implementována.", "Upozornění", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

private void Zákaz()
{
    (menuDalší.Enabled, tlačítkoDalší.Enabled, menuSpustitAnimaci.Enabled, časovač.Povolen) = (false, false, false, false);
}

        private void MenuDalší_Click(object sender, EventArgs e)
        {
            nalezeno = řešitel.NajdiDalšíŘešení(out řešení);
            if(!nalezeno)
            {
                MessageBox.Show(NPR.Neexistují, NPR.Vše, MessageBoxButtons.OK, MessageBoxIcon.Information);
                Invoke(new Action(Zákaz));
            }
            panel.Invalidate();
            Invoke(new Action<int, int>(NastavTitulek), kolik, řešitel.PočetNalezenýchŘešení);
        }

        private void MenuKonec_Click(object sender, EventArgs e)
        {
            Close();
            Application.Exit();
        }

        private void MenuOProgramu_Click(object sender, EventArgs e)
        {
            MessageBox.Show(NPR.OProgramuText1 + "\n" + NPR.OProgramuText2, NPR.OProgramuTitulek);
        }

        private void Okno_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (MessageBox.Show(NPR.Skončit, NPR.Dotaz, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                e.Cancel = true;
            }

        }

        private void Panel_Paint(object sender, PaintEventArgs e)
        {
            Graphics grafika = e.Graphics;
            if (nalezeno)
            {
                Pen pero = new Pen(Color.Black)
                {
                    Width = 1
                };
                var (deltax, deltay) = (panel.Width / kolik, panel.Height / kolik);
                NakresliŠachovnici(grafika, pero, deltax, deltay);
                NakresliDámy(grafika, pero, deltax, deltay);
            }
            else
            {
                grafika.Clear(Color.White);
            }
        }

        private void NakresliDámy(Graphics grafika, Pen pero, int deltax, int deltay)
        {
            pero.Width = 6;
            pero.Color = Color.Red;
            for (int i = 0; i < kolik; i++)
            {
                grafika.DrawEllipse(pero, i * deltax + deltax / 6, řešení[i] * deltay + deltay / 6, 2 * deltax / 3, 2 * deltay / 3);
            }
        }

        private void NakresliŠachovnici(Graphics grafika, Pen pero, int deltax, int deltay)
        {
            for (int i = 1; i < kolik; i++)
            {
                grafika.DrawLine(pero, 0, i * deltay, panel.Width, i * deltay);
                grafika.DrawLine(pero, i * deltax, 0, i * deltax, panel.Height);
            }
        }
        private void MenuVšechna_Click(object sender, EventArgs e)
        {
            (bool, bool, bool, bool) stav = (menuNová.Enabled, menuDalší.Enabled, menuVšechna.Enabled, tlačítkoDalší.Enabled);
            (menuNová.Enabled, menuDalší.Enabled, menuVšechna.Enabled) = (false, false, false);
            PřipravŘešitele();
            MessageBox.Show(string.Format(NPR.VýsledekText, kolik, řešitel.Řešení.Count(), řešitel.PočetVolání), NPR.Výsledek);
            (menuNová.Enabled, menuDalší.Enabled, menuVšechna.Enabled, tlačítkoDalší.Enabled) = stav;
        }

        private void Okno_Resize(object sender, EventArgs e)
        {
            panel.Invalidate();
        }

        private void MenuUložit_Click(object sender, EventArgs e)
        {
            if(dialogUložení.ShowDialog() == DialogResult.OK)
            {
                PřipravŘešitele();
                using (var výstup = new StreamWriter(dialogUložení.FileName))
                {
                    foreach(var řádka in řešitel.Řešení)
                    {
                        foreach(var i in řádka)
                        {
                            výstup.Write($"{i} ");
                        }
                        výstup.WriteLine();
                    }
                }
            }
        }

        private void MenuZastavitAnimaci_Click(object sender, EventArgs e) => časovač.Povolen = false;
        private void MenuSpustitAnimaci_Click(object sender, EventArgs e) => časovač.Povolen = true;
    }
}
